@@ -73,7 +73,7 @@ impl PublishCommand {
         }
         println!("\nSource files:");
         for file in src_files_added.iter().sorted() {
-            println!("  - {}", file);
+            println!("  - {file}");
         }
         println!("\nName: {}", config.name);
         println!("Version: {}", config.version);
@@ -702,8 +702,10 @@ fn release_metadata_as_erlang() {
 
 #[test]
 fn prevent_publish_local_dependency() {
-    let mut config = PackageConfig::default();
-    config.dependencies = [("provided".into(), Requirement::path("./path/to/package"))].into();
+    let config = PackageConfig {
+        dependencies: [("provided".into(), Requirement::path("./path/to/package"))].into(),
+        ..Default::default()
+    };
     assert_eq!(
         metadata_config(&config, &[], &[]),
         Err(Error::PublishNonHexDependencies {
@@ -714,12 +716,14 @@ fn prevent_publish_local_dependency() {
 
 #[test]
 fn prevent_publish_git_dependency() {
-    let mut config = PackageConfig::default();
-    config.dependencies = [(
-        "provided".into(),
-        Requirement::git("https://github.com/gleam-lang/gleam.git"),
-    )]
-    .into();
+    let config = PackageConfig {
+        dependencies: [(
+            "provided".into(),
+            Requirement::git("https://github.com/gleam-lang/gleam.git"),
+        )]
+        .into(),
+        ..Default::default()
+    };
     assert_eq!(
         metadata_config(&config, &[], &[]),
         Err(Error::PublishNonHexDependencies {
