@@ -10,13 +10,16 @@ use lsp_types::{
 use strum::IntoEnumIterator;
 
 use crate::{
-    ast::{self, Arg, CallArg, Definition, Function, Pattern, Publicity, TypedExpr},
+    ast::{
+        self, Arg, CallArg, Definition, Function, FunctionLiteralKind, Pattern, Publicity,
+        TypedExpr,
+    },
     build::Module,
-    io::{CommandExecutor, FileSystemReader, FileSystemWriter},
+    io::{BeamCompiler, CommandExecutor, FileSystemReader, FileSystemWriter},
     line_numbers::LineNumbers,
     type_::{
-        self, collapse_links, pretty::Printer, AccessorsMap, FieldMap, ModuleInterface,
-        PreludeType, Type, TypeConstructor, ValueConstructorVariant, PRELUDE_MODULE_NAME,
+        self, collapse_links, pretty::Printer, FieldMap, ModuleInterface, PreludeType,
+        RecordAccessor, Type, TypeConstructor, ValueConstructorVariant, PRELUDE_MODULE_NAME,
     },
     Result,
 };
@@ -60,7 +63,7 @@ fn sort_text(kind: CompletionKind, label: &str) -> String {
         CompletionKind::Prelude => 4,
         CompletionKind::ImportableModule => 5,
     };
-    format!("{}_{}", priority, label)
+    format!("{priority}_{label}")
 }
 
 // The form in which a type completion is needed in context.
@@ -93,6 +96,7 @@ where
     // IO to be supplied from outside of gleam-core
     IO: FileSystemReader
         + FileSystemWriter
+        + BeamCompiler
         + CommandExecutor
         + DownloadDependencies
         + MakeLocker
@@ -148,10 +152,10 @@ where
                 },
                 end: Position {
                     line: self.cursor_position.line,
-                    character: self.cursor_position.character + after.len() as u32,
+                    character: self.cursor_position.character,
                 },
             },
-            format!("{}{}", before, after),
+            format!("{before}{after}"),
         )
     }
 
@@ -719,13 +723,19 @@ where
         &'a self,
         importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
         type_: Arc<Type>,
-    ) -> Option<&AccessorsMap> {
+    ) -> Option<&HashMap<EcoString, RecordAccessor>> {
         let type_ = collapse_links(type_);
         match type_.as_ref() {
-            Type::Named { name, module, .. } => importable_modules
+            Type::Named {
+                name,
+                module,
+                inferred_variant,
+                ..
+            } => importable_modules
                 .get(module)
                 .and_then(|i| i.accessors.get(name))
-                .filter(|a| a.publicity.is_importable() || module == &self.module.name),
+                .filter(|a| a.publicity.is_importable() || module == &self.module.name)
+                .map(|a| a.accessors_for_variant(*inferred_variant)),
             _ => None,
         }
     }
@@ -737,9 +747,8 @@ where
             self.compiler.project_compiler.get_importable_modules(),
             type_,
         )
-        .map(|accessors_map| {
-            accessors_map
-                .accessors
+        .map(|accessors| {
+            accessors
                 .values()
                 .map(|accessor| field_completion(&accessor.label, accessor.type_.clone()))
                 .collect_vec()
@@ -1045,7 +1054,7 @@ impl<'ast> ast::visit::Visit<'ast> for LocalCompletion<'_> {
         &mut self,
         _: &'ast ast::SrcSpan,
         _: &'ast Arc<Type>,
-        _: &'ast bool,
+        _: &'ast FunctionLiteralKind,
         args: &'ast [ast::TypedArg],
         body: &'ast [ast::TypedStatement],
         _: &'ast Option<ast::TypeAst>,
