@@ -22,6 +22,33 @@ pub fn show_complete(code: &str, position: Position) -> String {
     str
 }
 
+fn apply_conversion(src: &str, completions: Vec<CompletionItem>, value: &str) -> String {
+    let completion = completions
+        .iter()
+        .find(|c| c.label == value.to_string())
+        .expect(&format!("no completion with value `{value}`"));
+
+    let mut edits = vec![];
+    if let Some(lsp_types::CompletionTextEdit::Edit(edit)) = &completion.text_edit {
+        edits.push(edit.clone());
+    }
+    apply_code_edit(src, edits)
+}
+
+#[macro_export]
+macro_rules! assert_apply_completion {
+    ($project:expr, $name:literal, $position:expr) => {
+        let src = $project.src;
+        let completions = completion($project, $position);
+        let output = format!(
+            "{}\n\n----- After applying completion -----\n{}",
+            show_complete(src, $position),
+            apply_conversion(src, completions, $name)
+        );
+        insta::assert_snapshot!(insta::internals::AutoName, output, src);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_completion {
     ($project:expr) => {
@@ -100,27 +127,27 @@ fn format_completion_results(completions: Vec<CompletionItem>) -> EcoString {
         buffer.push_str(&label);
 
         if let Some(kind) = kind {
-            write!(buffer, "\n  kind:   {:?}", kind).unwrap();
+            write!(buffer, "\n  kind:   {kind:?}").unwrap();
         }
 
         if let Some(detail) = detail {
-            write!(buffer, "\n  detail: {}", detail).unwrap();
+            write!(buffer, "\n  detail: {detail}").unwrap();
         }
 
         if let Some(sort_text) = sort_text {
-            write!(buffer, "\n  sort:   {}", sort_text).unwrap();
+            write!(buffer, "\n  sort:   {sort_text}").unwrap();
         }
 
         if let Some(label_details) = label_details {
             assert!(label_details.detail.is_none());
             if let Some(desc) = label_details.description {
-                write!(buffer, "\n  desc:   {}", desc).unwrap();
+                write!(buffer, "\n  desc:   {desc}").unwrap();
             }
         }
 
         if let Some(documentation) = documentation {
             let lsp_types::Documentation::MarkupContent(m) = documentation else {
-                panic!("unexpected docs in test {:?}", documentation);
+                panic!("unexpected docs in test {documentation:?}");
             };
             match m.kind {
                 lsp_types::MarkupKind::Markdown => (),
@@ -141,7 +168,7 @@ fn format_completion_results(completions: Vec<CompletionItem>) -> EcoString {
 
         if let Some(text_edit) = text_edit {
             let lsp_types::CompletionTextEdit::Edit(e) = text_edit else {
-                panic!("unexpected text edit in test {:?}", text_edit);
+                panic!("unexpected text edit in test {text_edit:?}");
             };
             buffer.push_str("\n  edits:");
             edit(&mut buffer, e);
@@ -1350,7 +1377,7 @@ fn internal_modules_from_same_package_are_included() {
 pub fn main() {
   0
 }";
-    let internal_name = format!("{}/internal", LSP_TEST_ROOT_PACKAGE_NAME);
+    let internal_name = format!("{LSP_TEST_ROOT_PACKAGE_NAME}/internal");
 
     assert_completion!(
         TestProject::for_source(code)
@@ -1613,6 +1640,41 @@ fn fun() {
     assert_completion!(TestProject::for_source(code), Position::new(8, 15));
 }
 
+#[test]
+fn completions_for_record_access_known_variant() {
+    let code = "
+type Wibble {
+  Wibble(a: Int, b: Int, c: Int, d: Int)
+  Wobble(z: Bool)
+}
+
+fn fun(some_wibble: Wibble) {
+  case some_wibble {
+    Wibble(..) as w -> w.a
+    Wobble(..) -> panic
+  }
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(8, 26));
+}
+
+#[test]
+fn completions_for_record_access_unknown_variant() {
+    let code = "
+type Wibble {
+  Wibble(a: Int, b: Int, c: Int, d: Int)
+  Wobble(a: Int, z: Bool)
+}
+
+fn fun(some_wibble: Wibble) {
+  some_wibble.a
+}
+";
+
+    assert_completion!(TestProject::for_source(code), Position::new(7, 15));
+}
+
 #[test]
 fn completions_for_record_labels() {
     let code = "
@@ -1753,3 +1815,44 @@ pub fn main(x: Int) {
 
     assert_completion!(TestProject::for_source(code), Position::new(3, 0));
 }
+
+// https://github.com/gleam-lang/gleam/issues/3833
+#[test]
+fn autocomplete_doesnt_delete_the_piece_of_code_that_comes_after() {
+    let code = "
+import list
+pub fn main(x: Int) {
+  list.list.filter([1, 2, 3], todo)
+}
+";
+    let list = "
+pub fn filter(_, _) { [] }
+pub fn map(_, _) { [] }
+";
+
+    assert_apply_completion!(
+        TestProject::for_source(code).add_dep_module("list", list),
+        "list.map",
+        Position::new(3, 7)
+    );
+}
+
+#[test]
+fn autocomplete_doesnt_delete_the_piece_of_code_that_comes_after_2() {
+    let code = "
+import list
+pub fn main(x: Int) {
+  list.mlist.filter([1, 2, 3], todo)
+}
+";
+    let list = "
+pub fn filter(_, _) { [] }
+pub fn map(_, _) { [] }
+";
+
+    assert_apply_completion!(
+        TestProject::for_source(code).add_dep_module("list", list),
+        "list.map",
+        Position::new(3, 8)
+    );
+}
