@@ -97,7 +97,8 @@ macro_rules! assert_js_module_infer {
 #[macro_export]
 macro_rules! assert_module_error {
     ($src:expr) => {
-        let output = $crate::type_::tests::module_error($src, vec![]);
+        let error = $crate::type_::tests::module_error($src, vec![]);
+        let output = format!("----- SOURCE CODE\n{}\n\n----- ERROR\n{}", $src, error);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -105,7 +106,8 @@ macro_rules! assert_module_error {
 #[macro_export]
 macro_rules! assert_internal_module_error {
     ($src:expr) => {
-        let output = $crate::type_::tests::internal_module_error($src, vec![]);
+        let error = $crate::type_::tests::internal_module_error($src, vec![]);
+        let output = format!("----- SOURCE CODE\n{}\n\n----- ERROR\n{}", $src, error);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -113,11 +115,12 @@ macro_rules! assert_internal_module_error {
 #[macro_export]
 macro_rules! assert_js_module_error {
     ($src:expr) => {
-        let output = $crate::type_::tests::module_error_with_target(
+        let error = $crate::type_::tests::module_error_with_target(
             $src,
             vec![],
             $crate::build::Target::JavaScript,
         );
+        let output = format!("----- SOURCE CODE\n{}\n\n----- ERROR\n{}", $src, error);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -125,7 +128,8 @@ macro_rules! assert_js_module_error {
 #[macro_export]
 macro_rules! assert_module_syntax_error {
     ($src:expr) => {
-        let output = $crate::type_::tests::syntax_error($src);
+        let error = $crate::type_::tests::syntax_error($src);
+        let output = format!("----- SOURCE CODE\n{}\n\n----- ERROR\n{}", $src, error);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -139,14 +143,19 @@ macro_rules! assert_error {
     };
 
     ($src:expr) => {
-        let error = $crate::type_::tests::compile_statement_sequence($src)
+        let (error, names) = $crate::type_::tests::compile_statement_sequence($src)
             .expect_err("should infer an error");
         let error = $crate::error::Error::Type {
+            names,
             src: $src.into(),
             path: camino::Utf8PathBuf::from("/src/one/two.gleam"),
             errors: error,
         };
-        let output = error.pretty_string();
+        let error_string = error.pretty_string();
+        let output = format!(
+            "----- SOURCE CODE\n{}\n\n----- ERROR\n{}",
+            $src, error_string
+        );
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -154,8 +163,20 @@ macro_rules! assert_error {
 #[macro_export]
 macro_rules! assert_with_module_error {
     (($name:expr, $module_src:literal), $src:expr $(,)?) => {
-        let output =
+        let error =
             $crate::type_::tests::module_error($src, vec![("thepackage", $name, $module_src)]);
+        let output = format!(
+            "----- SOURCE CODE
+-- {}.gleam
+{}
+
+-- main.gleam
+{}
+
+----- ERROR
+{}",
+            $name, $module_src, $src, error
+        );
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 
@@ -164,13 +185,28 @@ macro_rules! assert_with_module_error {
         ($name2:expr, $module_src2:literal),
         $src:expr $(,)?
     ) => {
-        let output = $crate::type_::tests::module_error(
+        let error = $crate::type_::tests::module_error(
             $src,
             vec![
                 ("thepackage", $name, $module_src),
                 ("thepackage", $name2, $module_src2),
             ],
         );
+        let output = format!(
+            "----- SOURCE CODE
+-- {}.gleam
+{}
+
+-- {}.gleam
+{}
+
+-- main.gleam
+{}
+
+----- ERROR
+{}",
+            $name, $module_src, $name2, $module_src2, $src, error
+        );
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -178,6 +214,7 @@ macro_rules! assert_with_module_error {
 fn get_warnings(
     src: &str,
     deps: Vec<DependencyModule<'_>>,
+    target: Target,
     gleam_version: Option<Range<Version>>,
 ) -> Vec<crate::warning::Warning> {
     let warnings = VectorWarningEmitterIO::default();
@@ -186,19 +223,21 @@ fn get_warnings(
         src,
         Some(Rc::new(warnings.clone())),
         deps,
-        Target::Erlang,
+        target,
         TargetSupport::NotEnforced,
         gleam_version,
-    );
+    )
+    .expect("Compilation should succeed");
     warnings.take().into_iter().collect_vec()
 }
 
 fn get_printed_warnings(
     src: &str,
     deps: Vec<DependencyModule<'_>>,
+    target: Target,
     gleam_version: Option<Range<Version>>,
 ) -> String {
-    print_warnings(get_warnings(src, deps, gleam_version))
+    print_warnings(get_warnings(src, deps, target, gleam_version))
 }
 
 fn print_warnings(warnings: Vec<crate::warning::Warning>) -> String {
@@ -212,13 +251,20 @@ fn print_warnings(warnings: Vec<crate::warning::Warning>) -> String {
 #[macro_export]
 macro_rules! assert_warnings_with_imports {
     ($(($name:literal, $module_src:literal)),+; $src:literal,) => {
-        let output = $crate::type_::tests::get_printed_warnings(
+        let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![
                 $(("thepackage", $name, $module_src)),*
             ],
+            crate::build::Target::Erlang,
             None
         );
+
+        let mut output = String::from("----- SOURCE CODE\n");
+        for (name, src) in [$(($name, $module_src)),*] {
+            output.push_str(&format!("-- {name}.gleam\n{src}\n\n"));
+        }
+        output.push_str(&format!("-- main.gleam\n{}\n\n----- WARNING\n{warning}", $src));
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -226,37 +272,76 @@ macro_rules! assert_warnings_with_imports {
 #[macro_export]
 macro_rules! assert_warning {
     ($src:expr) => {
-        let output = $crate::type_::tests::get_printed_warnings($src, vec![], None);
-        assert!(!output.is_empty());
+        let warning = $crate::type_::tests::get_printed_warnings($src, vec![], crate::build::Target::Erlang, None);
+        assert!(!warning.is_empty());
+        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 
     ($(($name:expr, $module_src:literal)),+, $src:expr) => {
-        let output = $crate::type_::tests::get_printed_warnings(
+        let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![$(("thepackage", $name, $module_src)),*],
+            crate::build::Target::Erlang,
             None
         );
-        assert!(!output.is_empty());
+        assert!(!warning.is_empty());
+        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 
     ($(($package:expr, $name:expr, $module_src:literal)),+, $src:expr) => {
-        let output = $crate::type_::tests::get_printed_warnings(
+        let warning = $crate::type_::tests::get_printed_warnings(
             $src,
             vec![$(($package, $name, $module_src)),*],
+            crate::build::Target::Erlang,
             None
         );
-        assert!(!output.is_empty());
+        assert!(!warning.is_empty());
+        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
 
+#[macro_export]
+macro_rules! assert_js_warning {
+    ($src:expr) => {
+        let warning = $crate::type_::tests::get_printed_warnings(
+            $src,
+            vec![],
+            crate::build::Target::JavaScript,
+            None,
+        );
+        assert!(!warning.is_empty());
+        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
+        insta::assert_snapshot!(insta::internals::AutoName, output, $src);
+    };
+}
+
+#[macro_export]
+macro_rules! assert_js_no_warnings {
+    ($src:expr) => {
+        let warning = $crate::type_::tests::get_printed_warnings(
+            $src,
+            vec![],
+            crate::build::Target::JavaScript,
+            None,
+        );
+        assert!(warning.is_empty());
+    };
+}
+
 #[macro_export]
 macro_rules! assert_warnings_with_gleam_version {
     ($gleam_version:expr, $src:expr$(,)?) => {
-        let output = $crate::type_::tests::get_printed_warnings($src, vec![], Some($gleam_version));
-        assert!(!output.is_empty());
+        let warning = $crate::type_::tests::get_printed_warnings(
+            $src,
+            vec![],
+            crate::build::Target::Erlang,
+            Some($gleam_version),
+        );
+        assert!(!warning.is_empty());
+        let output = format!("----- SOURCE CODE\n{}\n\n----- WARNING\n{}", $src, warning);
         insta::assert_snapshot!(insta::internals::AutoName, output, $src);
     };
 }
@@ -264,13 +349,14 @@ macro_rules! assert_warnings_with_gleam_version {
 #[macro_export]
 macro_rules! assert_no_warnings {
     ($src:expr $(,)?) => {
-        let warnings = $crate::type_::tests::get_warnings($src, vec![], None);
+        let warnings = $crate::type_::tests::get_warnings($src, vec![], crate::build::Target::Erlang, None);
         assert_eq!(warnings, vec![]);
     };
     ($(($package:expr, $name:expr, $module_src:literal)),+, $src:expr $(,)?) => {
         let warnings = $crate::type_::tests::get_warnings(
             $src,
             vec![$(($package, $name, $module_src)),*],
+            crate::build::Target::Erlang,
             None,
         );
         assert_eq!(warnings, vec![]);
@@ -279,7 +365,7 @@ macro_rules! assert_no_warnings {
 
 fn compile_statement_sequence(
     src: &str,
-) -> Result<Vec1<TypedStatement>, Vec1<crate::type_::Error>> {
+) -> Result<Vec1<TypedStatement>, (Vec1<crate::type_::Error>, Names)> {
     let ast = crate::parse::parse_statement_sequence(src).expect("syntax error");
     let mut modules = im::HashMap::new();
     let ids = UniqueIdGenerator::new();
@@ -289,16 +375,17 @@ fn compile_statement_sequence(
     // place.
     let _ = modules.insert(PRELUDE_MODULE_NAME.into(), build_prelude(&ids));
     let mut problems = Problems::new();
+    let mut environment = Environment::new(
+        ids,
+        "thepackage".into(),
+        None,
+        "themodule".into(),
+        Target::Erlang,
+        &modules,
+        TargetSupport::Enforced,
+    );
     let res = ExprTyper::new(
-        &mut Environment::new(
-            ids,
-            "thepackage".into(),
-            None,
-            "themodule".into(),
-            Target::Erlang,
-            &modules,
-            TargetSupport::Enforced,
-        ),
+        &mut environment,
         FunctionDefinition {
             has_body: true,
             has_erlang_external: false,
@@ -309,7 +396,7 @@ fn compile_statement_sequence(
     .infer_statements(ast);
     match Vec1::try_from_vec(problems.take_errors()) {
         Err(_) => Ok(res),
-        Ok(errors) => Err(errors),
+        Ok(errors) => Err((errors, environment.names)),
     }
 }
 
@@ -371,7 +458,7 @@ pub fn compile_module(
     src: &str,
     warnings: Option<Rc<dyn WarningEmitterIO>>,
     dep: Vec<DependencyModule<'_>>,
-) -> Result<TypedModule, Vec<crate::type_::Error>> {
+) -> Result<TypedModule, (Vec<crate::type_::Error>, Names)> {
     compile_module_with_opts(
         module_name,
         src,
@@ -391,7 +478,7 @@ pub fn compile_module_with_opts(
     target: Target,
     target_support: TargetSupport,
     gleam_version: Option<Range<Version>>,
-) -> Result<TypedModule, Vec<crate::type_::Error>> {
+) -> Result<TypedModule, (Vec<crate::type_::Error>, Names)> {
     let ids = UniqueIdGenerator::new();
     let mut modules = im::HashMap::new();
 
@@ -456,8 +543,8 @@ pub fn compile_module_with_opts(
 
     match inference_result {
         Outcome::Ok(ast) => Ok(ast),
-        Outcome::PartialFailure(_, errors) => Err(errors.into()),
-        Outcome::TotalFailure(error) => Err(error.into()),
+        Outcome::PartialFailure(ast, errors) => Err((errors.into(), ast.names)),
+        Outcome::TotalFailure(error) => Err((error.into(), Default::default())),
     }
 }
 
@@ -470,7 +557,7 @@ pub fn module_error_with_target(
     deps: Vec<DependencyModule<'_>>,
     target: Target,
 ) -> String {
-    let error = compile_module_with_opts(
+    let (error, names) = compile_module_with_opts(
         "themodule",
         src,
         None,
@@ -481,6 +568,7 @@ pub fn module_error_with_target(
     )
     .expect_err("should infer an error");
     let error = Error::Type {
+        names,
         src: src.into(),
         path: Utf8PathBuf::from("/src/one/two.gleam"),
         errors: Vec1::try_from_vec(error).expect("should have at least one error"),
@@ -497,7 +585,7 @@ pub fn internal_module_error_with_target(
     deps: Vec<DependencyModule<'_>>,
     target: Target,
 ) -> String {
-    let error = compile_module_with_opts(
+    let (error, names) = compile_module_with_opts(
         "thepackage/internal/themodule",
         src,
         None,
@@ -508,6 +596,7 @@ pub fn internal_module_error_with_target(
     )
     .expect_err("should infer an error");
     let error = Error::Type {
+        names,
         src: src.into(),
         path: Utf8PathBuf::from("/src/one/two.gleam"),
         errors: Vec1::try_from_vec(error).expect("should have at least one error"),
@@ -531,6 +620,7 @@ pub fn syntax_error(src: &str) -> String {
 fn field_map_reorder_test() {
     let int = |value: &str| UntypedExpr::Int {
         value: value.into(),
+        int_value: crate::parse::parse_int_value(value).unwrap(),
         location: SrcSpan { start: 0, end: 0 },
     };
 
@@ -666,6 +756,7 @@ fn infer_module_type_retention_test() {
         name: "ok".into(),
         definitions: vec![],
         type_info: (),
+        names: Default::default(),
     };
     let direct_dependencies = HashMap::from_iter(vec![]);
     let ids = UniqueIdGenerator::new();
@@ -1769,6 +1860,371 @@ fn record_update_generic_unannotated() {
     );
 }
 
+#[test]
+fn record_update_variant_inference() {
+    assert_module_infer!(
+        "
+pub type Shape {
+  Circle(cx: Int, cy: Int, radius: Int)
+  Square(x: Int, y: Int, width: Int, height: Int)
+}
+
+pub fn grow(shape) {
+  case shape {
+    Circle(radius:, ..) as circle -> Circle(..circle, radius: radius + 1)
+    Square(width:, height:, ..) as square -> Square(..square, width: width + 1, height: height + 1)
+  }
+}
+",
+        vec![
+            ("Circle", "fn(Int, Int, Int) -> Shape"),
+            ("Square", "fn(Int, Int, Int, Int) -> Shape"),
+            ("grow", "fn(Shape) -> Shape")
+        ]
+    );
+}
+
+#[test]
+fn record_access_variant_inference() {
+    assert_module_infer!(
+        "
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: Int)
+}
+
+pub fn get(wibble) {
+  case wibble {
+    Wibble(..) as w -> w.b
+    Wobble(..) as w -> w.c
+  }
+}
+",
+        vec![
+            ("Wibble", "fn(Int, Int) -> Wibble"),
+            ("Wobble", "fn(Int, Int) -> Wibble"),
+            ("get", "fn(Wibble) -> Int")
+        ]
+    );
+}
+
+#[test]
+fn local_variable_variant_inference() {
+    assert_module_infer!(
+        "
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: Int)
+}
+
+pub fn main() {
+  let always_wibble = Wibble(1, 2)
+  always_wibble.b
+}
+",
+        vec![
+            ("Wibble", "fn(Int, Int) -> Wibble"),
+            ("Wobble", "fn(Int, Int) -> Wibble"),
+            ("main", "fn() -> Int")
+        ]
+    );
+}
+
+#[test]
+fn record_update_variant_inference_for_original_variable() {
+    assert_module_infer!(
+        r#"
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: String)
+}
+
+pub fn update(wibble: Wibble) -> Wibble {
+  case wibble {
+    Wibble(..) -> Wibble(..wibble, a: 1)
+    Wobble(..) -> Wobble(..wibble, c: "hello")
+  }
+}
+"#,
+        vec![
+            ("Wibble", "fn(Int, Int) -> Wibble"),
+            ("Wobble", "fn(Int, String) -> Wibble"),
+            ("update", "fn(Wibble) -> Wibble")
+        ]
+    );
+}
+
+#[test]
+fn record_access_variant_inference_for_original_variable() {
+    assert_module_infer!(
+        "
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: Int)
+}
+
+pub fn get(wibble) {
+  case wibble {
+    Wibble(..) -> wibble.b
+    Wobble(..) -> wibble.c
+  }
+}
+",
+        vec![
+            ("Wibble", "fn(Int, Int) -> Wibble"),
+            ("Wobble", "fn(Int, Int) -> Wibble"),
+            ("get", "fn(Wibble) -> Int")
+        ]
+    );
+}
+
+#[test]
+fn variant_inference_for_imported_type() {
+    assert_infer_with_module!(
+        (
+            "wibble",
+            "
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: Int)
+}
+"
+        ),
+        "
+import wibble.{Wibble, Wobble}
+
+pub fn main(wibble) {
+  case wibble {
+    Wibble(..) -> Wibble(a: 1, b: wibble.b + 1)
+    Wobble(..) -> Wobble(..wibble, c: wibble.c - 4)
+  }
+}
+",
+        vec![("main", "fn(Wibble) -> Wibble")]
+    );
+}
+
+#[test]
+fn local_variable_variant_inference_for_imported_type() {
+    assert_infer_with_module!(
+        (
+            "wibble",
+            "
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: Int)
+}
+"
+        ),
+        "
+import wibble.{Wibble}
+
+pub fn main() {
+  let wibble = Wibble(4, 9)
+  Wibble(..wibble, b: wibble.b)
+}
+",
+        vec![("main", "fn() -> Wibble")]
+    );
+}
+
+#[test]
+fn record_update_variant_inference_fails_for_several_possible_variants() {
+    assert_module_error!(
+        "
+pub type Vector {
+  Vector2(x: Float, y: Float)
+  Vector3(x: Float, y: Float, z: Float)
+}
+
+pub fn increase_y(vector, by increase) {
+  case vector {
+    Vector2(y:, ..) as vector | Vector3(y:, ..) as vector ->
+      Vector2(..vector, y: y +. increase)
+  }
+}
+"
+    );
+}
+
+#[test]
+fn record_update_variant_inference_fails_for_several_possible_variants_on_subject_variable() {
+    assert_module_error!(
+        r#"
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: String)
+}
+
+pub fn update(wibble: Wibble) -> Wibble {
+  case wibble {
+    Wibble(..) | Wobble(..) -> Wibble(..wibble, a: 1)
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn type_unification_does_not_cause_false_positives_for_variant_matching() {
+    assert_module_error!(
+        r#"
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: String)
+}
+
+pub fn wibbler() { todo }
+
+pub fn main() {
+  let c = wibbler()
+
+  case todo {
+    Wibble(..) -> Wibble(..c, b: 1)
+    _ -> todo
+  }
+}
+"#
+    );
+}
+
+#[test]
+fn type_unification_does_not_allow_different_variants_to_be_treated_as_safe() {
+    assert_module_error!(
+        r#"
+pub type Wibble {
+  Wibble(a: Int, b: Int)
+  Wobble(a: Int, c: String)
+}
+
+pub fn main() {
+  let a = case todo {
+    Wibble(..) as b -> Wibble(..b, b: 1)
+    Wobble(..) as b -> Wobble(..b, c: "a")
+  }
+
+  a.b
+}
+"#
+    );
+}
+
+#[test]
+fn record_update_variant_inference_in_alternate_pattern_with_all_same_variants() {
+    assert_module_infer!(
+        r#"
+pub type Vector {
+  Vector2(x: Float, y: Float)
+  Vector3(x: Float, y: Float, z: Float)
+}
+
+pub fn increase_y(vector, by increase) {
+  case vector {
+    Vector2(y:, ..) as vector -> Vector2(..vector, y: y +. increase)
+    Vector3(y:, z: 12.3, ..) as vector | Vector3(y:, z: 15.0, ..) as vector ->
+      Vector3(..vector, y: y +. increase, z: 0.0)
+    _ -> panic as "Could not increase Y"
+  }
+}
+"#,
+        vec![
+            ("Vector2", "fn(Float, Float) -> Vector"),
+            ("Vector3", "fn(Float, Float, Float) -> Vector"),
+            ("increase_y", "fn(Vector, Float) -> Vector")
+        ]
+    );
+}
+
+#[test]
+fn variant_inference_does_not_escape_clause_scope() {
+    assert_module_error!(
+        "
+pub type Thingy {
+  A(a: Int)
+  B(x: Int, b: Int)
+}
+
+pub fn fun(x) {
+  case x {
+    A(..) -> x.a
+    B(..) -> x.b
+  }
+  x.b
+}
+"
+    );
+}
+
+#[test]
+// https://github.com/gleam-lang/gleam/issues/3797
+fn type_unification_removes_inferred_variant_in_tuples() {
+    assert_module_error!(
+        r#"
+pub type Either(a, b) {
+  Left(value: a)
+  Right(value: b)
+}
+
+fn a_or_b(_first: value, second: value) -> value {
+  second
+}
+
+pub fn main() {
+  let #(right) = a_or_b(#(Left(5)), #(Right("hello")))
+  Left(..right, value: 10)
+}
+"#
+    );
+}
+
+#[test]
+// https://github.com/gleam-lang/gleam/issues/3797
+fn type_unification_removes_inferred_variant_in_functions() {
+    assert_module_error!(
+        r#"
+pub type Either(a, b) {
+  Left(value: a)
+  Right(value: b)
+}
+
+fn a_or_b(_first: value, second: value) -> value {
+  second
+}
+
+pub fn main() {
+  let func = a_or_b(fn() { Left(1) }, fn() { Right("hello") })
+  Left(..func(), value: 10)
+}
+"#
+    );
+}
+
+#[test]
+// https://github.com/gleam-lang/gleam/issues/3797
+fn type_unification_removes_inferred_variant_in_nested_type() {
+    assert_module_error!(
+        r#"
+pub type Box(a) {
+  Box(inner: a)
+}
+
+pub type Either(a, b) {
+  Left(value: a)
+  Right(value: b)
+}
+
+fn a_or_b(_first: value, second: value) -> value {
+  second
+}
+
+pub fn main() {
+  let Box(inner) = a_or_b(Box(Left(1)), Box(Right("hello")))
+  Left(..inner, value: 10)
+}
+"#
+    );
+}
+
 #[test]
 fn module_constants() {
     assert_module_infer!(
@@ -2235,6 +2691,7 @@ fn assert_suitable_main_function_not_module_function() {
             literal: Constant::Int {
                 location: Default::default(),
                 value: "1".into(),
+                int_value: 1.into(),
             },
             implementations: Implementations {
                 gleam: true,
@@ -2452,3 +2909,83 @@ pub fn main() {
         vec![("main", "fn() -> Int")]
     );
 }
+
+#[test]
+fn labelled_argument_ordering() {
+    // https://github.com/gleam-lang/gleam/issues/3671
+    assert_module_infer!(
+        "
+type A { A }
+type B { B }
+type C { C }
+type D { D }
+
+fn wibble(a a: A, b b: B, c c: C, d d: D) {
+  Nil
+}
+
+pub fn main() {
+  wibble(A, C, D, b: B)
+  wibble(A, C, D, b: B)
+  wibble(B, C, D, a: A)
+  wibble(B, C, a: A, d: D)
+  wibble(B, C, d: D, a: A)
+  wibble(B, D, a: A, c: C)
+  wibble(B, D, c: C, a: A)
+  wibble(C, D, b: B, a: A)
+}
+",
+        vec![("main", "fn() -> Nil")]
+    );
+}
+
+#[test]
+fn variant_inference_allows_inference() {
+    // https://github.com/gleam-lang/gleam/pull/3647#issuecomment-2423146977
+    assert_module_infer!(
+        "
+pub type Wibble {
+  Wibble(a: Int)
+  Wobble(b: Int)
+}
+
+pub fn do_a_thing(wibble) {
+  case wibble {
+    Wibble(..) -> wibble.a
+    _ -> todo
+  }
+  wibble
+}
+",
+        vec![
+            ("Wibble", "fn(Int) -> Wibble"),
+            ("Wobble", "fn(Int) -> Wibble"),
+            ("do_a_thing", "fn(Wibble) -> Wibble")
+        ]
+    );
+}
+
+#[test]
+fn variant_inference_allows_inference2() {
+    // https://github.com/gleam-lang/gleam/pull/3647#issuecomment-2423146977
+    assert_module_infer!(
+        "
+pub type Box(a) {
+  Box(inner: a)
+  UnBox
+}
+
+pub fn rebox(box) {
+  case box {
+    Box(..) -> Box(box.inner + 1)
+    UnBox -> UnBox
+  }
+}
+",
+        vec![
+            ("Box", "fn(a) -> Box(a)"),
+            ("UnBox", "Box(a)"),
+            ("rebox", "fn(Box(Int)) -> Box(Int)")
+        ]
+    );
+}
